from .unpacking import (
    doublestarred,
    doublestarredpart,
    starred,
    starredpart,
    unpacking,
    unpackingpart,
)

star = starred
dblstar = doublestarred
unpk = unpacking

starpr = starredpart
dblstarpr = doublestarredpart
unpkpr = unpackingpart
